/*
 * Copyright 2022-2023 Sensaru GmbH
 */

#ifndef LIBC1_MODULE_SRC_ACL_USERACL_H
#define LIBC1_MODULE_SRC_ACL_USERACL_H

#include "AclCommon.h"
#include "UserCommon.h"
#include "../AssetId.h"

#include <unordered_set>
#include <homegear-base/Variable.h>

namespace C1Module {

struct UserAclRestPath {
  std::unordered_map<std::string, std::shared_ptr<UserAclRestPath>> segment;
  std::unordered_map<std::string, bool> methods;
};

struct UserModuleAcl {
  AclBool read = AclBool::kUndefined;
  AclBool write = AclBool::kUndefined;
  AclBool event = AclBool::kUndefined;
  AclBool is_admin = AclBool::kUndefined;
  std::unordered_set<std::string> rpc_methods;
};

typedef std::string AssetElementId;

struct AssetAccess {
  bool level_access = false;
  std::unordered_map<AssetElementId, std::shared_ptr<AssetAccess>> children;
};
typedef std::shared_ptr<AssetAccess> PAssetAccess;

enum class DatabaseFilterTypes {
  kUndefined,
  kAssets,
  kRoles,
  kAssetsAndRoles,
  kSingleRole,
  kAssetsAndSingleRole
};

class UserAcl {
 protected:
  uint32_t version_ = 1;
  std::unordered_map<std::string, UserModuleAcl> module_access_;
  BaseLib::PVariable serialized_rest_access_;
  std::shared_ptr<UserAclRestPath> rest_access_;
  BaseLib::PVariable serialized_asset_access_;
  PAssetAccess asset_access_;
  BaseLib::PVariable serialized_role_access_;
  std::unordered_set<int64_t> role_access_;

  std::shared_ptr<UserAclRestPath> GetRestAccessPathSegment(const std::vector<std::string> &path);
 public:
  UserAcl() = default;
  BaseLib::PVariable Serialize() const;
  static UserAcl Deserialize(const BaseLib::PVariable &acl_struct);

  std::unordered_map<std::string, UserModuleAcl> GetAccessibleModules() const;
  void GetDatabaseFilter(std::string &query, DatabaseFilterTypes filter_type, bool add_parent_elements = false) const;
  AclBool HasAssetAccess(const AssetId &asset_id) const;
  AclBool HasRoleAccess(int64_t role_id) const;
  AclBool HasRoleAccess(const std::unordered_set<int64_t> &role_ids) const;
};

typedef std::shared_ptr<UserAcl> PUserAcl;

}

#endif //LIBC1_MODULE_SRC_ACL_USERACL_H
