/*
 * Copyright 2022-2023 Sensaru GmbH
 */

#ifndef C1_MODULE_SRC_ASSETID_H_
#define C1_MODULE_SRC_ASSETID_H_

#include <homegear-base/Variable.h>
#include <homegear-base/HelperFunctions/HelperFunctions.h>

namespace C1Module {

enum class AssetIdLevel : uint32_t {
  kNone = 0,
  kPortfolio = 1,
  kEconomicUnit = 2,
  kProperty = 3,
  kAdministrationUnit = 4,
  kSubId = 5
};

struct AssetId {
  AssetIdLevel level = AssetIdLevel::kNone;
  std::string pf;
  std::string eu;
  std::string p;
  std::string au;
  std::string sid;

  static AssetId FromRpcArray(const BaseLib::PArray &value);
  [[nodiscard]] BaseLib::PVariable ToRpcArray() const;
  static AssetId FromString(const std::string &value);
  [[nodiscard]] std::string ToString() const;
  [[nodiscard]] static std::string GetLevelShortName(AssetIdLevel level);
  [[nodiscard]] AssetId GetParent();
  bool Empty() const;
  void Merge(const AssetId &child);
};

}
#endif
